package app

import (
	"context"
	"fmt"

	"coopcloud.tech/abra/cli/internal"
	"coopcloud.tech/abra/pkg/autocomplete"

	"coopcloud.tech/abra/pkg/client"
	"coopcloud.tech/abra/pkg/config"
	"coopcloud.tech/abra/pkg/dns"
	"coopcloud.tech/abra/pkg/formatter"
	"coopcloud.tech/abra/pkg/git"
	"coopcloud.tech/abra/pkg/lint"
	"coopcloud.tech/abra/pkg/recipe"
	"coopcloud.tech/abra/pkg/upstream/stack"
	"github.com/sirupsen/logrus"
	"github.com/urfave/cli"
)

var appDeployCommand = cli.Command{
	Name:      "deploy",
	Aliases:   []string{"d"},
	Usage:     "Deploy an app",
	ArgsUsage: "<domain>",
	Flags: []cli.Flag{
		internal.DebugFlag,
		internal.NoInputFlag,
		internal.ForceFlag,
		internal.ChaosFlag,
		internal.NoDomainChecksFlag,
		internal.DontWaitConvergeFlag,
		internal.OfflineFlag,
	},
	Before: internal.SubCommandBefore,
	Description: `
Deploy an app. It does not support incrementing the version of a deployed app,
for this you need to look at the "abra app upgrade <domain>" command.

You may pass "--force" to re-deploy the same version again. This can be useful
if the container runtime has gotten into a weird state.

Chaos mode ("--chaos") will deploy your local checkout of a recipe as-is,
including unstaged changes and can be useful for live hacking and testing new
recipes.
`,
	BashComplete: autocomplete.AppNameComplete,
	Action: func(c *cli.Context) error {
		app := internal.ValidateApp(c)
		stackName := app.StackName()

		if err := recipe.EnsureExists(app.Recipe); err != nil {
			logrus.Fatal(err)
		}

		if !internal.Chaos {
			if err := recipe.EnsureIsClean(app.Recipe); err != nil {
				logrus.Fatal(err)
			}

			if !internal.Offline {
				if err := recipe.EnsureUpToDate(app.Recipe); err != nil {
					logrus.Fatal(err)
				}
			}

			if err := recipe.EnsureLatest(app.Recipe); err != nil {
				logrus.Fatal(err)
			}
		}

		r, err := recipe.Get(app.Recipe, internal.Offline)
		if err != nil {
			logrus.Fatal(err)
		}

		if err := lint.LintForErrors(r); err != nil {
			logrus.Fatal(err)
		}

		logrus.Debugf("checking whether %s is already deployed", stackName)

		cl, err := client.New(app.Server)
		if err != nil {
			logrus.Fatal(err)
		}

		isDeployed, deployedVersion, err := stack.IsDeployed(context.Background(), cl, stackName)
		if err != nil {
			logrus.Fatal(err)
		}

		if isDeployed {
			if internal.Force || internal.Chaos {
				logrus.Warnf("%s is already deployed but continuing (--force/--chaos)", app.Name)
			} else {
				logrus.Fatalf("%s is already deployed", app.Name)
			}
		}

		isLatestHash := false
		version := deployedVersion
		if !internal.Chaos {
			catl, err := recipe.ReadRecipeCatalogue(internal.Offline)
			if err != nil {
				logrus.Fatal(err)
			}
			versions, err := recipe.GetRecipeCatalogueVersions(app.Recipe, catl)
			if err != nil {
				logrus.Fatal(err)
			}
			if len(versions) > 0 {
				version = versions[len(versions)-1]
				logrus.Debugf("choosing %s as version to deploy", version)
				if err := recipe.EnsureVersion(app.Recipe, version); err != nil {
					logrus.Fatal(err)
				}
			} else {
				head, err := git.GetRecipeHead(app.Recipe)
				if err != nil {
					logrus.Fatal(err)
				}
				isLatestHash = true
				version = formatter.SmallSHA(head.String())
				logrus.Warn("no versions detected, using latest commit")
			}
		}

		if version != "unknown" && !internal.Chaos && !isLatestHash {
			logrus.Debugf("choosing %s as version to deploy", version)
			if err := recipe.EnsureVersion(app.Recipe, version); err != nil {
				logrus.Fatal(err)
			}
		}

		if internal.Chaos {
			logrus.Warnf("chaos mode engaged")
			var err error
			version, err = recipe.ChaosVersion(app.Recipe)
			if err != nil {
				logrus.Fatal(err)
			}
		}

		abraShPath := fmt.Sprintf("%s/%s/%s", config.RECIPES_DIR, app.Recipe, "abra.sh")
		abraShEnv, err := config.ReadAbraShEnvVars(abraShPath)
		if err != nil {
			logrus.Fatal(err)
		}
		for k, v := range abraShEnv {
			app.Env[k] = v
		}

		composeFiles, err := config.GetAppComposeFiles(app.Recipe, app.Env)
		if err != nil {
			logrus.Fatal(err)
		}

		deployOpts := stack.Deploy{
			Composefiles: composeFiles,
			Namespace:    stackName,
			Prune:        false,
			ResolveImage: stack.ResolveImageAlways,
		}
		compose, err := config.GetAppComposeConfig(app.Name, deployOpts, app.Env)
		if err != nil {
			logrus.Fatal(err)
		}

		config.ExposeAllEnv(stackName, compose, app.Env)
		config.SetRecipeLabel(compose, stackName, app.Recipe)
		config.SetChaosLabel(compose, stackName, internal.Chaos)
		config.SetChaosVersionLabel(compose, stackName, version)
		config.SetUpdateLabel(compose, stackName, app.Env)

		if err := internal.DeployOverview(app, version, "continue with deployment?"); err != nil {
			logrus.Fatal(err)
		}

		if !internal.NoDomainChecks {
			domainName, ok := app.Env["DOMAIN"]
			if ok {
				if _, err = dns.EnsureDomainsResolveSameIPv4(domainName, app.Server); err != nil {
					logrus.Fatal(err)
				}
			} else {
				logrus.Warn("skipping domain checks as no DOMAIN=... configured for app")
			}
		} else {
			logrus.Warn("skipping domain checks as requested")
		}

		stack.WaitTimeout, err = config.GetTimeoutFromLabel(compose, stackName)
		if err != nil {
			logrus.Fatal(err)
		}
		logrus.Debugf("set waiting timeout to %d s", stack.WaitTimeout)

		if err := stack.RunDeploy(cl, deployOpts, compose, app.Name, internal.DontWaitConverge); err != nil {
			logrus.Fatal(err)
		}

		postDeployCmds, ok := app.Env["POST_DEPLOY_CMDS"]
		if ok && !internal.DontWaitConverge {
			logrus.Debugf("run the following post-deploy commands: %s", postDeployCmds)
			if err := internal.PostCmds(cl, app, postDeployCmds); err != nil {
				logrus.Fatalf("attempting to run post deploy commands, saw: %s", err)
			}
		}
		return nil
	},
}
