#!/usr/bin/env bash

setup_file(){
  load "$PWD/tests/integration/helpers/common"
  _common_setup
  _add_server
  _new_app
}

teardown_file(){
  _rm_app
  _rm_server
}

setup(){
  load "$PWD/tests/integration/helpers/common"
  _common_setup
}

@test "validate app argument" {
  run $ABRA app restore
  assert_failure
  assert_output --partial 'no app provided'

  run $ABRA app restore DOESNTEXIST
  assert_failure
  assert_output --partial 'cannot find app'
}

@test "error if missing service" {
  run $ABRA app restore "$TEST_APP_DOMAIN"
  assert_failure
  assert_output --partial 'missing <service>'

  run $ABRA app restore "$TEST_APP_DOMAIN" app
  assert_failure
  assert_output --partial 'missing <file>'
}

@test "error if file doesn't exist" {
  run $ABRA app restore "$TEST_APP_DOMAIN" app DOESNTEXIST
  assert_failure
  assert_output --partial "doesn't exist"
}

@test "retrieve recipe if missing" {
  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE"
  assert_success
  assert_not_exists "$ABRA_DIR/recipes/$TEST_RECIPE"

  run $ABRA app restore "$TEST_APP_DOMAIN" app DOESNTEXIST
  assert_failure
  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE"
}

# bats test_tags=slow
@test "detect labels if restore enabled" {
  run touch "$BATS_TMPDIR/foo.txt"
  assert_success
  assert_exists "$BATS_TMPDIR/foo.txt"

  run tar -cvf "$BATS_TMPDIR/foo.tar.gz" "$BATS_TMPDIR/foo.txt"
  assert_success
  assert_exists "$BATS_TMPDIR/foo.tar.gz"

  _deploy_app

  run $ABRA app restore "$TEST_APP_DOMAIN" app "$BATS_TMPDIR/foo.tar.gz" --debug
  assert_success
  assert_output --partial 'restore config detected'
  assert_output --partial 'detected pre-hook command'
  assert_output --partial 'detected post-hook command'

  _undeploy_app
}

# bats test_tags=slow
@test "no error if restore not enabled" {
  run sed -i '/backupbot.restore=.*/d' "$ABRA_DIR/recipes/$TEST_RECIPE/compose.yml"
  assert_success

  run touch "$BATS_TMPDIR/foo.txt"
  assert_success
  assert_exists "$BATS_TMPDIR/foo.txt"

  run tar -cvf "$BATS_TMPDIR/foo.tar.gz" "$BATS_TMPDIR/foo.txt"
  assert_success
  assert_exists "$BATS_TMPDIR/foo.tar.gz"

  run $ABRA app deploy "$TEST_APP_DOMAIN" --no-input --chaos
  assert_success

  run $ABRA app restore "$TEST_APP_DOMAIN" app "$BATS_TMPDIR/foo.tar.gz" --debug
  assert_success
  refute_output --partial 'restore config detected'
  refute_output --partial 'detected pre-hook command'
  refute_output --partial 'detected post-hook command'

  _undeploy_app

  _checkout_recipe "$TEST_RECIPE"
}

# bats test_tags=slow
@test "error if service doesn't exist" {
  run touch "$BATS_TMPDIR/foo.txt"
  assert_success
  assert_exists "$BATS_TMPDIR/foo.txt"

  run tar -cvf "$BATS_TMPDIR/foo.tar.gz" "$BATS_TMPDIR/foo.txt"
  assert_success
  assert_exists "$BATS_TMPDIR/foo.tar.gz"

  _deploy_app

  run $ABRA app restore "$TEST_APP_DOMAIN" DOESNTEXIST "$BATS_TMPDIR/foo.tar.gz" --debug
  assert_failure
  assert_output --partial 'no containers matching'

  _undeploy_app
}

# bats test_tags=slow
@test "restore backup" {
  run touch "$BATS_TMPDIR/foo.txt"
  assert_success
  assert_exists "$BATS_TMPDIR/foo.txt"

  run tar -cvf "$BATS_TMPDIR/foo.tar.gz" "$BATS_TMPDIR/foo.txt"
  assert_success
  assert_exists "$BATS_TMPDIR/foo.tar.gz"

  _deploy_app

  run $ABRA app restore "$TEST_APP_DOMAIN" app "$BATS_TMPDIR/foo.tar.gz" --debug
  assert_success
  assert_output --partial 'restore config detected'
  assert_output --partial 'detected pre-hook command'
  assert_output --partial 'detected post-hook command'

  run $ABRA app run "$TEST_APP_DOMAIN" app ls "$BATS_TMPDIR"
  assert_success
  assert_output --partial 'foo.txt'

  _undeploy_app
}
